@@ -1,114 +1,96 @@
-/* Copyright (C) 2008 Michael Jansen <kde@michael-jansen.biz>
+/*
+    SPDX-FileCopyrightText: 2008 Michael Jansen <kde@michael-jansen.biz>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "component.h"
 
-#include "globalshortcut.h"
 #include "globalshortcutcontext.h"
 #include "globalshortcutsregistry.h"
-#include "logging_p.h"
-#include <config-kglobalaccel.h>
 #include "kglobalaccel_interface.h"
+#include "logging_p.h"
+#include "sequencehelpers_p.h"
+#include <config-kglobalaccel.h>
 
-#include <QStringList>
 #include <QKeySequence>
+#include <QStringList>
 
 #if HAVE_X11
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
 #include <QX11Info>
 #endif
+#endif
 
-static QList<int> keysFromString(const QString &str)
+static QList<QKeySequence> keysFromString(const QString &str)
 {
-    QList<int> ret;
+    QList<QKeySequence> ret;
     if (str == QLatin1String("none")) {
         return ret;
     }
     const QStringList strList = str.split('\t');
     for (const QString &s : strList) {
-        int key = QKeySequence(s)[0];
-        if (key != -1) {     //sanity check just in case
+        QKeySequence key = QKeySequence(s);
+        if (!key.isEmpty()) { // sanity check just in case
             ret.append(key);
         }
     }
     return ret;
 }
 
-
-static QString stringFromKeys(const QList<int> &keys)
+static QString stringFromKeys(const QList<QKeySequence> &keys)
 {
     if (keys.isEmpty()) {
         return QStringLiteral("none");
     }
     QString ret;
-    for (int key : keys) {
-        ret.append(QKeySequence(key).toString());
+    for (const QKeySequence &key : keys) {
+        ret.append(key.toString());
         ret.append('\t');
     }
     ret.chop(1);
     return ret;
 }
 
-namespace KdeDGlobalAccel {
-
-Component::Component(
-            const QString &uniqueName,
-            const QString &friendlyName,
-            GlobalShortcutsRegistry *registry)
-    :   _uniqueName(uniqueName)
-        ,_friendlyName(friendlyName)
-        ,_registry(registry)
-    {
+namespace KdeDGlobalAccel
+{
+Component::Component(const QString &uniqueName, const QString &friendlyName, GlobalShortcutsRegistry *registry)
+    : _uniqueName(uniqueName)
+    , _friendlyName(friendlyName)
+    , _registry(registry)
+{
     // Make sure we do no get uniquenames still containing the context
-    Q_ASSERT(uniqueName.indexOf("|")==-1);
+    Q_ASSERT(uniqueName.indexOf("|") == -1);
 
-    // Register ourselve with the registry
-    if (_registry)
-        {
+    // Register ourselves with the registry
+    if (_registry) {
         _registry->addComponent(this);
-        }
-
-    QString DEFAULT=QStringLiteral("default");
-    createGlobalShortcutContext(DEFAULT, QStringLiteral("Default Context"));
-    _current = _contexts.value(DEFAULT);
     }
 
+    QString DEFAULT = QStringLiteral("default");
+    createGlobalShortcutContext(DEFAULT, QStringLiteral("Default Context"));
+    _current = _contexts.value(DEFAULT);
+}
 
 Component::~Component()
-    {
-    // Remove ourselve from the registry
-    if (_registry)
-        {
+{
+    // Remove ourselves from the registry
+    if (_registry) {
         _registry->takeComponent(this);
-        }
+    }
 
     // We delete all shortcuts from all contexts
     qDeleteAll(_contexts);
-    }
+}
 
-
-bool Component::activateGlobalShortcutContext(
-        const QString &uniqueName)
-    {
-    if (!_contexts.value(uniqueName))
-        {
+bool Component::activateGlobalShortcutContext(const QString &uniqueName)
+{
+    if (!_contexts.value(uniqueName)) {
         createGlobalShortcutContext(uniqueName, "TODO4");
         return false;
-        }
+    }
 
     // Deactivate the current contexts shortcuts
     deactivateShortcuts();
@@ -117,58 +99,47 @@ bool Component::activateGlobalShortcutContext(
     _current = _contexts.value(uniqueName);
 
     return true;
-    }
-
+}
 
 void Component::activateShortcuts()
-    {
-    for (GlobalShortcut *shortcut : qAsConst(_current->_actions))
-        {
+{
+    for (GlobalShortcut *shortcut : std::as_const(_current->_actions)) {
         shortcut->setActive();
-        }
     }
+}
 
-
-QList<GlobalShortcut*> Component::allShortcuts(const QString &contextName) const
-    {
+QList<GlobalShortcut *> Component::allShortcuts(const QString &contextName) const
+{
     GlobalShortcutContext *context = _contexts.value(contextName);
-    if (context)
-        {
+    if (context) {
         return context->_actions.values();
-        }
-    else
-        {
-        return QList<GlobalShortcut*> ();
-        }
+    } else {
+        return QList<GlobalShortcut *>();
     }
-
+}
 
 QList<KGlobalShortcutInfo> Component::allShortcutInfos(const QString &contextName) const
-    {
+{
     GlobalShortcutContext *context = _contexts.value(contextName);
-    if (!context)
-        {
+    if (!context) {
         return QList<KGlobalShortcutInfo>();
-        }
-
-    return context->allShortcutInfos();
     }
 
+    return context->allShortcutInfos();
+}
 
 bool Component::cleanUp()
-    {
+{
     bool changed = false;
 
     const auto actions = _current->_actions;
-    for (GlobalShortcut *shortcut : actions)
-        {
+    for (GlobalShortcut *shortcut : actions) {
         qCDebug(KGLOBALACCELD) << _current->_actions.size();
-        if (!shortcut->isPresent())
-            {
+        if (!shortcut->isPresent()) {
             changed = true;
             shortcut->unRegister();
-            }
         }
+    }
 
     if (changed) {
         _registry->writeSettings();
@@ -176,62 +147,52 @@ bool Component::cleanUp()
     }
 
     return changed;
-    }
+}
 
-
-bool Component::createGlobalShortcutContext(
-        const QString &uniqueName,
-        const QString &friendlyName)
-    {
-    if (_contexts.value(uniqueName))
-        {
+bool Component::createGlobalShortcutContext(const QString &uniqueName, const QString &friendlyName)
+{
+    if (_contexts.value(uniqueName)) {
         qCDebug(KGLOBALACCELD) << "Shortcut Context " << uniqueName << "already exists for component " << _uniqueName;
         return false;
-        }
+    }
     _contexts.insert(uniqueName, new GlobalShortcutContext(uniqueName, friendlyName, this));
     return true;
-    }
-
+}
 
 GlobalShortcutContext *Component::currentContext()
-    {
+{
     return _current;
-    }
-
+}
 
 QDBusObjectPath Component::dbusPath() const
-    {
+{
     QString dbusPath = _uniqueName;
     // Clean up for dbus usage: any non-alphanumeric char should be turned into '_'
     const int len = dbusPath.length();
-    for ( int i = 0; i < len; ++i )
-        {
-        if ( !dbusPath[i].isLetterOrNumber() || dbusPath[i].unicode() >= 0x7F )
+    for (int i = 0; i < len; ++i) {
+        if (!dbusPath[i].isLetterOrNumber() || dbusPath[i].unicode() >= 0x7F) {
             // DBus path can only contain ASCII characters
             dbusPath[i] = QLatin1Char('_');
         }
-    // QDBusObjectPath could be a little bit easier to handle :-)
-    return QDBusObjectPath( _registry->dbusPath().path() + "component/" + dbusPath);
     }
-
+    // QDBusObjectPath could be a little bit easier to handle :-)
+    return QDBusObjectPath(_registry->dbusPath().path() + "component/" + dbusPath);
+}
 
 void Component::deactivateShortcuts(bool temporarily)
-    {
-    for (GlobalShortcut *shortcut : qAsConst(_current->_actions))
-        {
-        if (temporarily
-                && uniqueName() == QLatin1String("kwin")
-                && shortcut->uniqueName() == QLatin1String("Block Global Shortcuts"))
-            {
+{
+    for (GlobalShortcut *shortcut : std::as_const(_current->_actions)) {
+        if (temporarily //
+            && uniqueName() == QLatin1String("kwin") //
+            && shortcut->uniqueName() == QLatin1String("Block Global Shortcuts")) {
             continue;
-            }
-        shortcut->setInactive();
         }
+        shortcut->setInactive();
     }
+}
 
-
-void Component::emitGlobalShortcutPressed( const GlobalShortcut &shortcut )
-    {
+void Component::emitGlobalShortcutPressed(const GlobalShortcut &shortcut)
+{
 #if HAVE_X11
     // pass X11 timestamp
     long timestamp = QX11Info::appTime();
@@ -247,245 +208,209 @@ void Component::emitGlobalShortcutPressed( const GlobalShortcut &shortcut )
 #endif
 
     // Make sure it is one of ours
-    if (shortcut.context()->component() != this)
-        {
+    if (shortcut.context()->component() != this) {
         // In production mode do nothing
         return;
-        }
-
-    emit globalShortcutPressed(
-            shortcut.context()->component()->uniqueName(),
-            shortcut.uniqueName(),
-            timestamp);
     }
 
+    Q_EMIT globalShortcutPressed(shortcut.context()->component()->uniqueName(), shortcut.uniqueName(), timestamp);
+}
+
 void Component::invokeShortcut(const QString &shortcutName, const QString &context)
-    {
-        GlobalShortcut *shortcut = getShortcutByName(shortcutName, context);
-        if (shortcut) emitGlobalShortcutPressed(*shortcut);
+{
+    GlobalShortcut *shortcut = getShortcutByName(shortcutName, context);
+    if (shortcut) {
+        emitGlobalShortcutPressed(*shortcut);
     }
+}
 
 QString Component::friendlyName() const
-    {
-    if (_friendlyName.isEmpty())
+{
+    if (_friendlyName.isEmpty()) {
         return _uniqueName;
+    }
     return _friendlyName;
-    }
+}
 
+GlobalShortcut *Component::getShortcutByKey(const QKeySequence &key, KGlobalAccel::MatchType type) const
+{
+    return _current->getShortcutByKey(key, type);
+}
 
-GlobalShortcut *Component::getShortcutByKey(int key) const
-    {
-    return _current->getShortcutByKey(key);
-    }
-
-
-QList<GlobalShortcut *> Component::getShortcutsByKey(int key) const
-    {
-    QList <GlobalShortcut *> rc;
-    for (GlobalShortcutContext *context : qAsConst(_contexts))
-        {
-        GlobalShortcut *sc = context->getShortcutByKey(key);
-        if (sc) rc.append(sc);
+QList<GlobalShortcut *> Component::getShortcutsByKey(const QKeySequence &key, KGlobalAccel::MatchType type) const
+{
+    QList<GlobalShortcut *> rc;
+    for (GlobalShortcutContext *context : std::as_const(_contexts)) {
+        GlobalShortcut *sc = context->getShortcutByKey(key, type);
+        if (sc) {
+            rc.append(sc);
         }
-    return rc;
     }
-
+    return rc;
+}
 
 GlobalShortcut *Component::getShortcutByName(const QString &uniqueName, const QString &context) const
-    {
-    if (!_contexts.value(context))
-        {
+{
+    if (!_contexts.value(context)) {
         return nullptr;
-        }
+    }
 
     return _contexts.value(context)->_actions.value(uniqueName);
-    }
-
+}
 
 QStringList Component::getShortcutContexts() const
-    {
+{
     return _contexts.keys();
-    }
-
+}
 
 bool Component::isActive() const
-    {
+{
     // The component is active if at least one of it's global shortcuts is
     // present.
-    for (GlobalShortcut *shortcut : qAsConst(_current->_actions))
-        {
-        if (shortcut->isPresent()) return true;
+    for (GlobalShortcut *shortcut : std::as_const(_current->_actions)) {
+        if (shortcut->isPresent()) {
+            return true;
         }
-    return false;
     }
+    return false;
+}
 
-
-bool Component::isShortcutAvailable(
-        int key,
-        const QString &component,
-        const QString &context) const
-    {
-    qCDebug(KGLOBALACCELD) << QKeySequence(key).toString() << component;
+bool Component::isShortcutAvailable(const QKeySequence &key, const QString &component, const QString &context) const
+{
+    qCDebug(KGLOBALACCELD) << key.toString() << component;
 
     // if this component asks for the key. only check the keys in the same
     // context
-    if (component==uniqueName())
-        {
+    if (component == uniqueName()) {
         const auto actions = shortcutContext(context)->_actions;
-        for (GlobalShortcut *sc : actions)
-            {
-            if (sc->keys().contains(key)) return false;
+        for (GlobalShortcut *sc : actions) {
+            if (matchSequences(key, sc->keys())) {
+                return false;
             }
         }
-    else
-        {
-        for (GlobalShortcutContext *ctx : qAsConst(_contexts))
-            {
-            for (GlobalShortcut *sc : qAsConst(ctx->_actions))
-                {
-                if (sc->keys().contains(key)) return false;
+    } else {
+        for (GlobalShortcutContext *ctx : std::as_const(_contexts)) {
+            for (GlobalShortcut *sc : std::as_const(ctx->_actions)) {
+                if (matchSequences(key, sc->keys())) {
+                    return false;
                 }
             }
         }
-    return true;
     }
+    return true;
+}
 
-GlobalShortcut *Component::registerShortcut(const QString &uniqueName, const QString &friendlyName, const QString &shortcutString, const QString &defaultShortcutString)
-    {
+GlobalShortcut *
+Component::registerShortcut(const QString &uniqueName, const QString &friendlyName, const QString &shortcutString, const QString &defaultShortcutString)
+{
     // The shortcut will register itself with us
-    GlobalShortcut *shortcut = new GlobalShortcut(
-            uniqueName,
-            friendlyName,
-            currentContext());
+    GlobalShortcut *shortcut = new GlobalShortcut(uniqueName, friendlyName, currentContext());
 
-    const QList<int> keys = keysFromString(shortcutString);
+    const QList<QKeySequence> keys = keysFromString(shortcutString);
     shortcut->setDefaultKeys(keysFromString(defaultShortcutString));
     shortcut->setIsFresh(false);
-    QList<int> newKeys = keys;
-    for (int key : keys)
-        {
-        if (key != 0)
-            {
-            if (GlobalShortcutsRegistry::self()->getShortcutByKey(key))
-                {
+    QList<QKeySequence> newKeys = keys;
+    for (const QKeySequence &key : keys) {
+        if (!key.isEmpty()) {
+            if (GlobalShortcutsRegistry::self()->getShortcutByKey(key)) {
                 // The shortcut is already used. The config file is
                 // broken. Ignore the request.
                 newKeys.removeAll(key);
-                qCWarning(KGLOBALACCELD) << "Shortcut found twice in kglobalshortcutsrc."<<key;
-                }
+                qCWarning(KGLOBALACCELD) << "Shortcut found twice in kglobalshortcutsrc." << key;
             }
         }
+    }
     shortcut->setKeys(keys);
     return shortcut;
-    }
-
+}
 
 void Component::loadSettings(KConfigGroup &configGroup)
-    {
+{
     // GlobalShortcutsRegistry::loadSettings handles contexts.
     const auto listKeys = configGroup.keyList();
-    for (const QString &confKey : listKeys)
-        {
+    for (const QString &confKey : listKeys) {
         const QStringList entry = configGroup.readEntry(confKey, QStringList());
-        if (entry.size() != 3)
-            {
+        if (entry.size() != 3) {
             continue;
-            }
+        }
 
         GlobalShortcut *shortcut = registerShortcut(confKey, entry[2], entry[0], entry[1]);
         if (configGroup.name().endsWith(QLatin1String(".desktop"))) {
             shortcut->setIsPresent(true);
         }
-        }
     }
-
+}
 
 void Component::setFriendlyName(const QString &name)
-    {
+{
     _friendlyName = name;
-    }
+}
 
-
-GlobalShortcutContext *Component::shortcutContext( const QString &contextName )
-    {
+GlobalShortcutContext *Component::shortcutContext(const QString &contextName)
+{
     return _contexts.value(contextName);
-    }
+}
 
-
-GlobalShortcutContext const *Component::shortcutContext( const QString &contextName ) const
-    {
+GlobalShortcutContext const *Component::shortcutContext(const QString &contextName) const
+{
     return _contexts.value(contextName);
-    }
+}
 
-
-QStringList Component::shortcutNames( const QString &contextName) const
-    {
+QStringList Component::shortcutNames(const QString &contextName) const
+{
     GlobalShortcutContext *context = _contexts.value(contextName);
-    if (!context)
-        {
+    if (!context) {
         return QStringList();
-        }
+    }
 
     return context->_actions.keys();
-    }
-
+}
 
 QString Component::uniqueName() const
-    {
+{
     return _uniqueName;
-    }
-
+}
 
 void Component::unregisterShortcut(const QString &uniqueName)
-    {
+{
     // Now wrote all contexts
-    for( GlobalShortcutContext *context : qAsConst(_contexts))
-        {
-        if (context->_actions.value(uniqueName))
-            {
+    for (GlobalShortcutContext *context : std::as_const(_contexts)) {
+        if (context->_actions.value(uniqueName)) {
             delete context->takeShortcut(context->_actions.value(uniqueName));
-            }
         }
     }
+}
 
-
-void Component::writeSettings(KConfigGroup& configGroup) const
-    {
+void Component::writeSettings(KConfigGroup &configGroup) const
+{
     // If we don't delete the current content global shortcut
     // registrations will never not deleted after forgetGlobalShortcut()
     configGroup.deleteGroup();
 
-
     // Now write all contexts
-    for( GlobalShortcutContext *context : qAsConst(_contexts))
-        {
+    for (GlobalShortcutContext *context : std::as_const(_contexts)) {
         KConfigGroup contextGroup;
 
-        if (context->uniqueName() == QLatin1String("default"))
-            {
+        if (context->uniqueName() == QLatin1String("default")) {
             contextGroup = configGroup;
             // Write the friendly name
             contextGroup.writeEntry("_k_friendly_name", friendlyName());
-            }
-        else
-            {
+        } else {
             contextGroup = KConfigGroup(&configGroup, context->uniqueName());
             // Write the friendly name
             contextGroup.writeEntry("_k_friendly_name", context->friendlyName());
-            }
+        }
 
         // qCDebug(KGLOBALACCELD) << "writing group " << _uniqueName << ":" << context->uniqueName();
 
-        for (const GlobalShortcut *shortcut : qAsConst(context->_actions))
-            {
+        for (const GlobalShortcut *shortcut : std::as_const(context->_actions)) {
             // qCDebug(KGLOBALACCELD) << "writing" << shortcut->uniqueName();
 
             // We do not write fresh shortcuts.
             // We do not write session shortcuts
-            if (shortcut->isFresh() || shortcut->isSessionShortcut())
-                {
+            if (shortcut->isFresh() || shortcut->isSessionShortcut()) {
                 continue;
-                }
+            }
             // qCDebug(KGLOBALACCELD) << "really writing" << shortcut->uniqueName();
 
             QStringList entry(stringFromKeys(shortcut->keys()));
@@ -493,8 +418,8 @@ void Component::writeSettings(KConfigGroup& configGroup) const
             entry.append(shortcut->friendlyName());
 
             contextGroup.writeEntry(shortcut->uniqueName(), entry);
-            }
         }
     }
+}
 
 } // namespace KdeDGlobalAccel
