@@ -1,21 +1,9 @@
-/* This file is part of the KDE libraries
-    Copyright (C) 2001,2002 Ellis Whitehead <ellis@kde.org>
-    Copyright (C) 2015 Martin Gräßlin <mgraesslin@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2001, 2002 Ellis Whitehead <ellis@kde.org>
+    SPDX-FileCopyrightText: 2015 Martin Gräßlin <mgraesslin@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KGLOBALACCEL_INTERFACE_H
@@ -27,6 +15,8 @@
 
 class GlobalShortcutsRegistry;
 
+#define KGlobalAccelInterface_iid "org.kde.kglobalaccel5.KGlobalAccelInterface"
+
 /**
  * Abstract interface for plugins to implement
  */
@@ -36,7 +26,7 @@ class KGLOBALACCELPRIVATE_EXPORT KGlobalAccelInterface : public QObject
 
 public:
     explicit KGlobalAccelInterface(QObject *parent);
-    virtual ~KGlobalAccelInterface();
+    ~KGlobalAccelInterface() override;
 
 public:
     /**
@@ -80,6 +70,6 @@ private:
     QScopedPointer<Private> d;
 };
 
-Q_DECLARE_INTERFACE(KGlobalAccelInterface, "org.kde.kglobalaccel5.KGlobalAccelInterface")
+Q_DECLARE_INTERFACE(KGlobalAccelInterface, KGlobalAccelInterface_iid)
 
 #endif
